part of jaguar_auth.session;

/// JWT based session manager with `authorization` header as transport mechanism
///
/// Stores all session as JWT token on `authorization` header
class JwtHeaderSession extends Object
    with _BaseJwtSession
    implements SessionManager {
  final JwtConfig config;

  final JwtValidationConfig validationConfig;

  JwtHeaderSession(this.config,
      {this.validationConfig: const JwtValidationConfig()});

  /// Parses session from the given [request]
  Future<Session> parse(Request request) async {
    String authHeaderStr = request.headers.value(HttpHeaders.AUTHORIZATION);

    final jwtToken =
        new AuthHeaderItem.fromHeaderBySchema(authHeaderStr, kScheme);

    // Is the token present?
    if (jwtToken is! AuthHeaderItem) {
      return new Session.newSession({});
    }

    return decodeJwt(jwtToken.credentials);
  }

  /// Writes [response] with session details
  Future<Response> write(Request request, Response resp) async {
    final String oldHeader = resp.headers.value(HttpHeaders.AUTHORIZATION);
    final headers = new AuthHeaders.fromHeaderStr(oldHeader);
    headers
        .addItem(new AuthHeaderItem(kScheme, encodeJwt(await request.session)));
    resp.headers.set(HttpHeaders.AUTHORIZATION, headers.toString());

    return resp;
  }

  static const String kScheme = 'Bearer';
}
