part of jaguar_auth.session;

/// JWT based session manager with Cookie as transport mechanism
///
/// Stores all session as JWT token on a Cookie
class JwtCookieSession extends Object
    with _BaseJwtSession
    implements SessionManager {
  /// Name of the cookie on which session data is stored
  final String cookieName;

  final JwtConfig config;

  final JwtValidationConfig validationConfig;

  JwtCookieSession(this.config,
      {this.validationConfig: const JwtValidationConfig(),
      this.cookieName = 'session'});

  /// Parses session from the given [request]
  Future<Session> parse(Request request) async {
    for (Cookie cook in request.cookies) {
      if (cook.name == cookieName) return decodeJwt(cook.value);
    }
    return new Session.newSession({});
  }

  /// Writes [response] with session details
  Future<Response> write(Request request, Response resp) async {
    final session = await request.session;
    final cook = new Cookie(cookieName, encodeJwt(session));
    resp.cookies.add(cook);
    return resp;
  }
}
